//Copyright © 2015-2017, Verizon and/or its Licensors. All rights reserved.

#import "SignInViewController.h"
#import "AppConstants.h"
#import "UIViewController+Alert.h"
#ifdef HUBBLE_MEDIA_BROWSER
#import "MediaGroupViewController.h"
#endif
#ifdef KEPLER_FILE_BROWSER
#import "MetaDataViewController.h"
#import "MetaDataTabBarController.h"
#endif
#import "PCClientCredentials+Addition.h"

@interface SignInViewController () <UITextFieldDelegate>

@property (nonatomic, assign) BOOL didPresentSignInScreen;

- (IBAction)signInButtonClicked:(id)sender;

@end

@implementation SignInViewController

- (void)viewDidLoad {
    [super viewDidLoad];
}

- (void)viewDidAppear:(BOOL)animated {

    [super viewDidAppear:animated];
    
    if (!self.didPresentSignInScreen) {

        self.didPresentSignInScreen = YES;
        [self signInButtonClicked:nil];
    }
}

- (void)didReceiveMemoryWarning {
    [super didReceiveMemoryWarning];
    // Dispose of any resources that can be recreated.
}

#ifdef HUBBLE_MEDIA_BROWSER
- (void)pushDashboardScreen {
    UIStoryboard *storyBoard = [UIStoryboard storyboardWithName:@"Main" bundle:nil];
    MediaGroupViewController *controller = (MediaGroupViewController*)[storyBoard instantiateViewControllerWithIdentifier: @"MediaGroupViewController"];
    controller.navigationItem.hidesBackButton = YES;
    [self.navigationController pushViewController:controller animated:YES];
}
#endif

#ifdef KEPLER_FILE_BROWSER
- (void)pushMetaDataScreen {
    
   UIStoryboard *storyBoard = [UIStoryboard storyboardWithName:@"Main" bundle:nil];
    UITabBarController *tabBarController = (MetaDataTabBarController*)[storyBoard instantiateViewControllerWithIdentifier: @"MetaDataTabBarController"];
    tabBarController.navigationItem.hidesBackButton = YES;
    [self presentViewController:tabBarController animated:YES completion:nil];
}
#endif

- (NSString *)keyChainID {

#if TARGET_IPHONE_SIMULATOR
    return @"PCSampleApp.GenericKeychainID";
#else
    return [[NSBundle mainBundle] bundleIdentifier];
#endif
}

- (IBAction)signInButtonClicked:(id)sender {
    
    kNilCredentials creds = [PCClientCredentials checkForNilCredsWithClientID:Client_ID
                                                                 clientSecret:Client_Secret
                                                                  redirectURL:Redirect_URL];
    NSMutableArray *nilCredentials = [NSMutableArray new];
    
    if (creds & kNIlClientIDCredential) {
        [nilCredentials addObject:@"Empty ClientID"];
    }
    
    if (creds & kNilClientSecretCredential) {
        [nilCredentials addObject:@"Empty ClientSecret"];
    }
    
    if (creds & kNilRedirectURLCredential) {
        [nilCredentials addObject:@"Empty redirectURL"];
    }
    
    if (nilCredentials.count) {
       
        NSString *errorMessage = [nilCredentials componentsJoinedByString:@"\n"];
        [self showOKAlertWithTitle:NSLocalizedString(@"Empty credentials", nilCredentials)
                        andMessage:errorMessage
                  attributedString:nil
                          okAction:nil];
    } else {
        
        /*
         @Description -
         This API is used to show the login screen and have the authentication for SDK. Once user name and password filled, the authentication is done/failed and result will come with completion block.
         
         @parameters -
         PCClientCredentials : This parameters contains attribute such as client ID, client Secret, Redirect URL, KeychainID. All parameters are required to call this API. If user will not provide required parameter, API will throw an NSInvalidArgumentException and user have to handle it.
         UIViewController : This parameter is used to present OAuth2 screen. This parameter is required to call this API.
         
         @CompletionHandler -
         This completion block gets called once OAuth2 process is done. completion block contains below parameters.
         PCAuthenticationResponse : This object stores access token and related credentials information. If user is successfully signed in, this object is stored in keychain and this will remain in keychain until user forcefully calls logout.
         PCOAuthLoginPageResponse : This object stores the response recieved from the OAuth 2 login page.
         PCAPIQueryResponse : Stores all the server response and error for this API.
         
         @Steps to call API -
         * Initialize the PCClientCredentials object with required parameters. Explained in parameters section.
         * Initialize PCAPISDK and call the authenticateClient: API.
         * Get the result in completion block.
         */
        
        // Intialize the PCClientCredentials object.
        PCClientCredentials *clientCredentials = [[PCClientCredentials alloc] initWithClientID:Client_ID
                                                                                  clientSecret:Client_Secret
                                                                                   redirectURL:Redirect_URL
                                                                                    keychainID:[self keyChainID]];
        
        __weak SignInViewController *blocksafeSelf = self;
        
        // Initilize the PCAPISDK and call the API.
        [[PCAPISDK sharedInstance] authenticateClient:clientCredentials fromViewController:self.navigationController withCompletionHandler:^(PCAuthenticationResponse *authenticationCredentials, PCOAuthLoginPageResponse *OAuthLoginPageResponse, PCAPIQueryResponse *response) {
            // Get the result in this block.
            if ([authenticationCredentials isAuthenticated]) {
#ifdef HUBBLE_MEDIA_BROWSER
                [blocksafeSelf pushDashboardScreen];
#endif
#ifdef KEPLER_FILE_BROWSER
                [blocksafeSelf pushMetaDataScreen];
#endif
            } else if ([OAuthLoginPageResponse.response isEqualToString:@"access_denied"]) {
           
                [blocksafeSelf showOKAlertWithTitle:NSLocalizedString(@"Error", nil)
                                andMessage:NSLocalizedString(@"Access denied", nil)
                          attributedString:nil
                                  okAction:nil];
            } else if ([OAuthLoginPageResponse.response isEqualToString:@"cancelled"]) {
                // do nothing
            } else if (response.urlResponse.statusCode == PCAPIResponseCodeServiceUnavailable) {
                
                [blocksafeSelf showServerUnavailableAlert];
            } else {
            
                [blocksafeSelf showOperationFailedAlert:NSLocalizedString(@"Login", nil)];
            }
        }];
    }
}

@end
