//Copyright © 2015-2017, Verizon and/or its Licensors. All rights reserved.

#import <UIKit/UIKit.h>

@class PCResource, PCAPIQueryResponse, UploadInfo;

@interface UploadObj : NSObject

@property (nonatomic, strong) UIImage *thumbnailImage;
@property (nonatomic, strong) NSString *fileName;
@property (nonatomic, strong) NSString *filePath;
@property (nonatomic, strong) NSString *destinationFolder;
@property (nonatomic, strong) UploadInfo *uploadInfo;

@end

@protocol UploadDelegate <NSObject>

- (void)uploadManagerDidAddFileToUploadQueue:(UploadObj *)file;
- (void)uploadManagerDidBeginUploadingFile:(UploadObj *)file;
- (void)uploadManagerDidProgressedUploadingFile:(UploadObj *)file progress:(CGFloat)progress state:(PCUploadAPIState)state;
- (void)uploadManagerDidFinishUploadingFile:(UploadObj *)file result:(PCResource *)metadata response:(PCAPIQueryResponse *)response;

@end

/*! uploadmanager upload files serially. It also informs the upload progress of individual files through it's delegate methods
 *
 */
@interface UploadManager : NSObject

// indicates if upload process is in progress, set to NO once all upload is done.
@property (nonatomic, readonly, assign) BOOL isUploadInProgress;
// total files those were ether uploaded or failed.
@property (nonatomic, readonly, assign) int totalFilesDone;
// total number of files queued to uploaded.
@property (nonatomic, readonly, assign) int totalFilesToBeUploaded;
// upload progress of current file.
@property (nonatomic, readonly, assign) float progress;
@property (nonatomic, weak) id<UploadDelegate> delegate;
@property (nonatomic, readonly) UploadObj *uploadingFile;

// sigleton method.
+ (UploadManager *)sharedInstance;

/*!
 * call this method to add UploadObj file in queue to perform upload operation.
 * \params file stores information for UploadObj.
 */
- (void)addFileToUploadQueue:(UploadObj *)file;

// Clean and cancel all upload operation in queue.
- (void)cleanUpload;

@end
