//Copyright © 2015-2017, Verizon and/or its Licensors. All rights reserved.

#import "ContactSearchViewController.h"
#import "ContactTableViewCell.h"
#import "PCContact+Additions.h"
#import "ImageDownloadView.h"
#import "ContactDetailViewController.h"
#import "UIViewController+Contact.h"
#import "ContactsDisplayViewController.h"
#import "UIViewController+Spinner.h"
#import "UIViewController+Alert.h"


@interface ContactSearchViewController ()

@property (nonatomic, strong) NSArray *contacts;
@property (nonatomic, strong) NSArray *searchResults;

@end

@implementation ContactSearchViewController

- (void)viewDidLoad {

    [super viewDidLoad];
}

#pragma mark -

- (NSArray *)dataSource {

    return self.searchResults;
}


#pragma mark - UISearchResultsUpdating

- (void)updateSearchResultsForSearchController:(UISearchController *)searchController {
    
    NSString *searchText = searchController.searchBar.text;
    
    NSString *strippedString = [searchText stringByTrimmingCharactersInSet:[NSCharacterSet whitespaceCharacterSet]];
    NSArray *searchItems = nil;
    if (strippedString.length > 0) {
        searchItems = [strippedString componentsSeparatedByString:@" "];
    }
    
    NSMutableArray *andMatchPredicates = [NSMutableArray array];
    
    for (NSString *searchString in searchItems) {
        
        NSMutableArray *searchItemsPredicate = [NSMutableArray array];
        
        NSPredicate *predicate = [NSPredicate predicateWithFormat:@"firstName BEGINSWITH[cd] %@", searchString];
        [searchItemsPredicate addObject:predicate];
        
        
        predicate = [NSPredicate predicateWithFormat:@"lastName BEGINSWITH[cd] %@", searchString];
        [searchItemsPredicate addObject:predicate];

        predicate = [NSPredicate predicateWithFormat:@"orgName BEGINSWITH[cd] %@", searchString];
        [searchItemsPredicate addObject:predicate];
        predicate = [NSPredicate predicateWithFormat:@"nickname BEGINSWITH[cd] %@", searchString];
        [searchItemsPredicate addObject:predicate];

        predicate = [NSPredicate predicateWithFormat:@"ANY emails.email BEGINSWITH[cd] %@", searchString];
        [searchItemsPredicate addObject:predicate];
        
        predicate = [NSPredicate predicateWithFormat:@"ANY telphones.number.description BEGINSWITH[cd] %@", searchString];
        [searchItemsPredicate addObject:predicate];
        
        
        NSCompoundPredicate *orMatchPredicates = [NSCompoundPredicate orPredicateWithSubpredicates:searchItemsPredicate];
        [andMatchPredicates addObject:orMatchPredicates];
    }
    
    NSCompoundPredicate *finalCompoundPredicate = [NSCompoundPredicate andPredicateWithSubpredicates:andMatchPredicates];
    NSMutableArray *searchResults = [[self.contacts filteredArrayUsingPredicate:finalCompoundPredicate] mutableCopy];
    
    self.searchResults = searchResults;
    [self.tableView reloadData];
}

#pragma mark -
#pragma mark ContactDetailViewController delegates

-(void) updateOldContact:(PCContact *)oldContact withUpdatedContact:(PCContact *)updatedContact {
    
    NSMutableArray *contacts = [NSMutableArray arrayWithArray:self.contacts];
    NSInteger index=[contacts indexOfObject:oldContact];
    [contacts replaceObjectsAtIndexes:[NSIndexSet indexSetWithIndex:index] withObjects:[NSArray arrayWithObject:updatedContact]];
    self.contacts = [NSArray arrayWithArray:contacts];
    [(ContactsDisplayViewController*) self.searchMasterController updateOldContact:oldContact withUpdatedContact:updatedContact];
    [self.tableView reloadData];
}
#pragma mark -
#pragma mark ContactAction delegates

- (void)didBeginDoingContactAction:(ContactAction)action {
    [self showSpinner];
}

- (void)didFinishDoingContactAction:(ContactAction)action onContact:(id)contact result:(id)result response:(id)response {
    
    PCAPIQueryResponse *APIResponse = (PCAPIQueryResponse *)response;
    
    [self hideSpinner];
    
    switch (APIResponse.urlResponse.statusCode) {
        case PCAPIResponseCodeResourceCreated:
        case PCAPIResponseCodeNoContent:
        case PCAPIResponseCodeOK: {
            
            switch (action) {
                case deleteContactAction: {
                    NSMutableArray *contacts = [NSMutableArray arrayWithArray:self.searchResults];
                    [contacts removeObject:contact];
                    self.searchResults = [NSArray arrayWithArray:contacts];
                     [(ContactsDisplayViewController*) self.searchMasterController deleteContact:contact];
                    [self.tableView reloadData];
                }
                    break;
                default:
                    break;
            }
        }
            break;
        case PCAPIResponseCodeUnauthorized:
            [self showUnAuthorizedAlert];
            break;
        case PCAPIResponseCodeServiceUnavailable:
            [self showServerUnavailableAlert];
            break;
        default:
            [self showOperationFailedAlert:[self stringForContactAction:action]];
            break;
    }
}

@end
