//Copyright © 2015-2017, Verizon and/or its Licensors. All rights reserved.

#import "CreateContactViewController.h"
#import "UIViewController+Contact.h"
#import "CNContact+Additions.h"
#import "UIViewController+Alert.h"
#import "UIViewController+Spinner.h"
#import "PCAPISDKCommonHeader.h"

@interface CreateContactViewController ()

@end

@interface MediaGroupViewController ()
-(void) refreshContact;
@end

@implementation CreateContactViewController

- (void)viewDidLoad {
    [super viewDidLoad];
    // Do any additional setup after loading the view.
}

#pragma mark -
#pragma CNContactViewControllerDelegate

- (BOOL)contactViewController:(CNContactViewController *)viewController shouldPerformDefaultActionForContactProperty:(CNContactProperty *)property {
    return YES;
}

/*!
 * @abstract Called when the view has completed.
 * @discussion If creating a new contact, the new contact added to the contacts list will be passed.
 * If adding to an existing contact, the existing contact will be passed.
 * @note It is up to the delegate to dismiss the view controller.
 */
- (void)contactViewController:(CNContactViewController *)viewController didCompleteWithContact:(nullable CNContact *)contact {
    
    if (contact) {
        
        /*
         @Description :
         This API is used to create a contact in the account. Once the create process is succeed/failed result will come with completion block.
         
         @Parameters :
         PCCreateContactAPIParams:
         * This class stores all the parameters required to create a contact.
         * For providing access token to SDK user have to logged In.
         * If user will not provide required parameter, API will throw an NSInvalidArgumentException and user have to handle it.
         * More information about parameters you will find in PCCreateContactAPIParams.h class.
         
         @CompletionHandler :
         This completion block gets called once the create process is succeed/failed. completion block contains below parameter.
         PCContact : Stores created contact details.
         PCAPIQueryResponse : Stores all the server response and error for this API.
         
         @To create a contact in the account, you must take the following steps:
         * Include VerizonCloudAPI.framework header file in your class.
         #import <VerizonCloudAPI/VerizonCloudAPI.h>
         * Initalize the PCCreateContactAPIParams class and provide the required parameter.
         * Initalize the PCAPISDK object and call the createContact:withCompletionHandler: API.
         * User can also able to initalize PCCreateContactAPIParams using PCContact through initWithPCContact method.
         * Perform the createContact:withCompletionHandler: API call using the code sample below.
         * Get the result (succeed or failed) in completion block.
         * If session is no longer valid , call the logout API which remove Access token stored in keychain. Again call authenticate:fromViewController:withCompletionHandler: API to get Access token.
         */
        
        // Initilize the PCAPISDK and call the API.
        PCCreateContactAPIParams *params = [[PCCreateContactAPIParams alloc] initWithPCContact:[contact getPCContact]];
        
        __weak CreateContactViewController *blockself = self;
        [self showSpinner];
        
        [[PCAPISDK sharedInstance] createContact:params withCompletionHandler:^(PCContact *contact, PCAPIQueryResponse *response) {
        [self hideSpinner];
        PCAPIQueryResponse *APIResponse = (PCAPIQueryResponse *)response;
            
            switch (APIResponse.urlResponse.statusCode) {
                case PCAPIResponseCodeResourceCreated:
                {
                    [_mediaViewController refreshContact];
                    [blockself.navigationController popViewControllerAnimated:YES];
                }
                    break;
                case PCAPIResponseCodeUnauthorized:
                    [blockself showUnAuthorizedAlert];
                    break;
                case PCAPIResponseCodeServiceUnavailable:
                    [blockself showServerUnavailableAlert];
                    break;
                case PCAPIResponseCodeBadRequest:
                    [blockself showOperationFailedAlert:[[response.jsonResponse objectForKey:@"errors"] objectForKey:@"error_description"]];
                default:
                    [blockself showOperationFailedAlert:[self stringForContactAction:createContactAction]];
                    break;
            }
        }];
         
    }else{
        [self.navigationController popViewControllerAnimated:YES];
    }
}

- (void)didReceiveMemoryWarning {
    [super didReceiveMemoryWarning];
    // Dispose of any resources that can be recreated.
}

/*
#pragma mark - Navigation

// In a storyboard-based application, you will often want to do a little preparation before navigation
- (void)prepareForSegue:(UIStoryboardSegue *)segue sender:(id)sender {
    // Get the new view controller using [segue destinationViewController].
    // Pass the selected object to the new view controller.
}
*/

@end
