//Copyright © 2015-2017, Verizon and/or its Licensors. All rights reserved.

#import "PCContact+Additions.h"
#import "ImageDownloadView.h"
#import <UIKit/UIKit.h>

@implementation PCContact (Additions)

- (NSString *)getAnyPhoneNumber {

    if (self.telphones.count) {
        
        PCTelContactType *telephone = [self.telphones objectAtIndex:0];
        return [NSString stringWithFormat:@"%@", telephone.number];
    }
    return nil;
}

- (NSString *)getAnyEmail {
    
    if (self.emails.count) {
        
        PCEmailContactType *email = [self.emails objectAtIndex:0];
        return [NSString stringWithFormat:@"%@", email.email];
    }
    return nil;
}

- (NSString *)getDisplaytext {

    if (self.firstName.length) {
       
        return self.firstName;
        
    } else if (self.lastName.length) {
       
        return self.lastName;
        
    } else if (self.orgName.length) {
        
        return self.orgName;
        
    } else if ([self getAnyEmail].length) {
       
        return [self getAnyEmail];
        
    } else if ([self getAnyPhoneNumber].length) {
       
        return [self getAnyPhoneNumber];
        
    }
    
    return NSLocalizedString(@"Unknown", nil);
}

- (CNMutableContact *)getCNContact {

    CNMutableContact *CNContact = [CNMutableContact new];
    CNContact.givenName = self.firstName;
    CNContact.familyName = self.lastName;
    CNContact.organizationName = self.orgName;
    CNContact.jobTitle = self.jobTitle;
    CNContact.namePrefix = self.namePrefix;
    CNContact.nickname = self.nickname;
    CNContact.note = self.note;
    CNContact.middleName = self.middleName;
    CNContact.nameSuffix = self.nameSuffix;
    CNContact.departmentName = self.orgUnit;
    CNContact.birthday = [self convertServerDateStringToNSDateComponents:self.birthday];
    
    CNContact.imageData = UIImagePNGRepresentation([UIImage imageNamed:@"Contact"]);
    
    if (self.telphones.count) {
        
        NSMutableArray *CNTelephoneArray = [NSMutableArray new];
        for (PCTelContactType *telephone in self.telphones) {
            
            CNLabeledValue *CNtelephone = [CNLabeledValue labeledValueWithLabel:telephone.type value:[CNPhoneNumber phoneNumberWithStringValue:[NSString stringWithFormat:@"%@", telephone.number]]];

            [CNTelephoneArray addObject:CNtelephone];
        }
        
        CNContact.phoneNumbers = CNTelephoneArray;
    }
    
    if (self.emails.count) {
        
        NSMutableArray *CNEmailArray = [NSMutableArray new];
        
        for (PCEmailContactType *email in self.emails) {
            
            CNLabeledValue *CNEmail = [CNLabeledValue labeledValueWithLabel:email.type value:email.email];
            [CNEmailArray addObject:CNEmail];
        }
        
        CNContact.emailAddresses = CNEmailArray;
    }
    
    if (self.imArray.count) {
        
        NSMutableArray *CNIMArray = [NSMutableArray new];
        
        for (PCInstantMessageType *im in self.imArray) {
            CNLabeledValue *CNIm = [CNLabeledValue labeledValueWithLabel:im.type value:[[CNInstantMessageAddress alloc] initWithUsername:im.address service:im.type]];
            [CNIMArray addObject:CNIm];
        }
        
        CNContact.instantMessageAddresses = CNIMArray;
    }
    
    if (self.addresses.count) {
        
        NSMutableArray *CNAddressArray = [NSMutableArray new];
        
        for (PCAddressType *address in self.addresses) {
            CNMutablePostalAddress *postalAddress = [CNMutablePostalAddress new];
            postalAddress.street = address.street;
            postalAddress.city = address.city;
            postalAddress.postalCode = address.pobox;
            postalAddress.country = address.country;
            postalAddress.state = address.state;

            CNLabeledValue *CNAddress = [CNLabeledValue labeledValueWithLabel:address.type value:postalAddress];
            [CNAddressArray addObject:CNAddress];
        }
        
        CNContact.postalAddresses = CNAddressArray;
    }
    
    return CNContact;
}

- (NSMutableDictionary *)getDictionary {
  
    NSMutableDictionary *dictionary = [NSMutableDictionary new];
    if (self.firstName) {
        [dictionary setObject:self.firstName forKey:@"firstname"];
    }
    if (self.lastName) {
        [dictionary setObject:self.lastName forKey:@"lastname"];
    }
    if (self.nickname) {
        [dictionary setObject:self.nickname forKey:@"nickname"];
    }
    if (self.version) {
        [dictionary setObject:self.version forKey:@"version"];
    }
    if (self.namePrefix) {
        [dictionary setObject:self.namePrefix forKey:@"nameprefix"];
    }
    if (self.nameSuffix) {
        [dictionary setObject:self.nameSuffix forKey:@"namesuffix"];
    }
    if (self.ID) {
        [dictionary setObject:self.ID forKey:@"id"];
    }
    if (self.source) {
         [dictionary setObject:self.source forKey:@"source"];
    }
    if (self.created) {
        [dictionary setObject:self.created forKey:@"created"];
    }
    if (self.modified) {
        [dictionary setObject:self.modified forKey:@"modified"];
    }
    if (self.jobTitle) {
        [dictionary setObject:self.jobTitle forKey:@"jobtitle"];
    }
    if (self.orgName) {
        [dictionary setObject:self.orgName forKey:@"orgname"];
    }
    if (self.photo) {
        [dictionary setObject:self.photo forKey:@"photo"];
    }
    if (self.photoReference) {
        [dictionary setObject:self.photoReference forKey:@"photoreference"];
    }
    if (self.note) {
        [dictionary setObject:self.note forKey:@"note"];
    }
    if (self.birthday) {
         [dictionary setObject:self.birthday forKey:@"birthday"];
    }
    if (self.anniversary) {
        [dictionary setObject:self.anniversary forKey:@"anniversary"];
    }
    if (self.middleName) {
         [dictionary setObject:self.middleName forKey:@"middlename"];
    }
    if (self.spouse) {
         [dictionary setObject:self.spouse forKey:@"spouse"];
    }
    if (self.children) {
        [dictionary setObject:self.children forKey:@"children"];
    }
    if (self.orgUnit) {
        [dictionary setObject:self.orgUnit forKey:@"orgunit"];
    }
    if (self.profession) {
        [dictionary setObject:self.profession forKey:@"profession"];
    }
    if (self.assistant) {
        [dictionary setObject:self.assistant forKey:@"assistant"];
    }
    if (self.relationship) {
        [dictionary setObject:self.relationship forKey:@"relationship"];
    }
    if (self.gender) {
        [dictionary setObject:self.gender forKey:@"gender"];
    }
    if (self.maritalStatus) {
        [dictionary setObject:self.maritalStatus forKey:@"maritalStatus"];
    }
    
    [dictionary setObject:[NSNumber numberWithBool:self.favorite] forKey:@"favorite"];
    [dictionary setObject:[NSNumber numberWithBool:self.incaseOfEmergency] forKey:@"incaseofemergency"];

    if (self.telphones.count) {
        
        NSMutableArray *dictTelephoneArray = [NSMutableArray new];
        for (PCTelContactType *telephone in self.telphones) {
            
            NSMutableDictionary *telDict = [NSMutableDictionary new];
            
            if (telephone.type) {
                [telDict setObject:telephone.type forKey:@"type"];
            }
            if (telephone.indx) {
                [telDict setObject:telephone.indx forKey:@"indx"];
            }
            if (telephone.preference) {
                [telDict setObject:telephone.preference forKey:@"preference"];
            }
            if (telephone.number) {
                [telDict setObject:telephone.number forKey:@"number"];
            }
            [dictTelephoneArray addObject:telDict];
        }
         [dictionary setObject:dictTelephoneArray forKey:@"tel"];
        
    }
    
    if (self.emails.count) {
        
        NSMutableArray *dictEmailArray = [NSMutableArray new];
        
        for (PCEmailContactType *email in self.emails) {
            NSMutableDictionary *emailDict = [NSMutableDictionary new];
            
            if (email.type) {
                [emailDict setObject:email.type forKey:@"type"];
            }
            if (email.indx) {
                [emailDict setObject:email.indx forKey:@"indx"];
            }
            if (email.preference) {
                [emailDict setObject:email.preference forKey:@"preference"];
            }
            if (email.email) {
               [emailDict setObject:email.email forKey:@"address"];
            }
            
            [dictEmailArray addObject:emailDict];
        }
        
         [dictionary setObject:dictEmailArray forKey:@"email"];
    }
    
    if (self.imArray.count) {
        
        NSMutableArray *dictIMArray = [NSMutableArray new];
        
        for (PCInstantMessageType *im in self.imArray) {
            
            NSMutableDictionary *imDict = [NSMutableDictionary new];
            
            if (im.type) {
                [imDict setObject:im.type forKey:@"type"];
            }
            if (im.preference) {
               [imDict setObject:im.preference forKey:@"preference"];
            }
            if (im.address) {
                [imDict setObject:im.address forKey:@"address"];
            }
            
            [dictIMArray addObject:imDict];
            
        }
            [dictionary setObject:dictIMArray forKey:@"im"];
    }
    
    if (self.addresses.count) {
        
        NSMutableArray *dictAddressArray = [NSMutableArray new];
        
        for (PCAddressType *address in self.addresses) {
            
            NSMutableDictionary *addressDict = [NSMutableDictionary new];
            
            if (address.type) {
                [addressDict setObject:address.type forKey:@"type"];
            }
            if (address.preference) {
                [addressDict setObject:address.preference forKey:@"preference"];
            }
            if (address.indx) {
                [addressDict setObject:address.indx forKey:@"indx"];
            }
            if (address.pobox) {
                 [addressDict setObject:address.pobox forKey:@"pobox"];
            }
            if (address.street) {
                 [addressDict setObject:address.street forKey:@"street"];
            }
            if (address.apartment) {
               [addressDict setObject:address.apartment forKey:@"apartment"];
            }
            if (address.city) {
                [addressDict setObject:address.city forKey:@"city"];
            }
            if (address.state) {
                [addressDict setObject:address.state forKey:@"state"];
            }
            if (address.zipcode) {
                [addressDict setObject:address.zipcode forKey:@"zipcode"];
            }
            if (address.country) {
                 [addressDict setObject:address.country forKey:@"country"];
            }
            
            [dictAddressArray addObject:addressDict];
        }
        
        [dictionary setObject:dictAddressArray forKey:@"address"];
    }
    
    return dictionary;
}


- (NSString *)thumbnailCachePath {
    
    NSArray *paths = NSSearchPathForDirectoriesInDomains(NSDocumentDirectory, NSUserDomainMask, YES);
    NSString *documentsDirectory = [paths objectAtIndex:0];
    NSString *thumbnailImagePath = [documentsDirectory stringByAppendingPathComponent:[NSString stringWithFormat:@"Thumbs_%@.jpg", self.photoReference]];
    return thumbnailImagePath;
}

- (UIImage *)defaultPic {
    
    return [UIImage imageNamed:@"Contact"];
}

- (void)setAvatarForImageView:(ImageDownloadView *)imageView withCompletionHandler:(void (^)(BOOL didSetImage))completion {

    if (self.photoReference.length) {
        
        imageView.imageID = self.photoReference; // imageID needs to be set to encounter cell reusability feature
        
        UIImage *thumbnailImage = [UIImage imageWithContentsOfFile:[self thumbnailCachePath]];
        
        if (thumbnailImage) {
            
            imageView.image = thumbnailImage;
            completion(YES);

        } else {
            
            // Intialize the PCThumbnailAPIParams object.
            PCThumbnailAPIParams *params = [[PCThumbnailAPIParams alloc] init];
            params.th = 320;
            params.tw = 320;
            params.contentToken = self.photoReference;
            
            imageView.image = [self defaultPic];
            
            __weak PCContact *blocksafeSelf = self;
            /*
             @Description :
             This API is used to download thumbnail image of resource. Once the download process is succeed/failed result will come with completion block.
             
             @Parameters :
             PCThumbnailAPIParams:
             * This class stores all the parameters required to download thumbnail image of resource. It contains attributes such as contentToken, size, th and tw.
             * For providing access token to SDK user have to logged In.
             * contentToken and Either 'size' or ('th' and 'tw') is required parameter to call this API.
             * value for size are (Thumbnail size: 'xs' (24x24), 's' (64x64), or 'm' (128x128).
             * If user will not provide required parameter, API will throw an NSInvalidArgumentException and user have to handle it.
             * More information about parameters you will find in PCThumbnailAPIParams class.
             
             @CompletionHandler :
             This completion block gets called once the download process is succeed/failed. completion block contains below parameter.
             PCAPIQueryResponse : Stores all the server response and error for this API. dowloaded result comes in downloadedURL attribute.
             
             @To download thumbnail image of resource, you must take the following steps:
             * Include VerizonCloudAPI.framework header file in your class.
             #import <VerizonCloudAPI/VerizonCloudAPI.h>
             * Initalize the PCThumbnailAPIParams class and provide the required parameter.
             * Initalize the PCAPISDK object and call the downloadThumbnail:withCompletionHandler: API.
             * Perform the downloadThumbnail:withCompletionHandler: API call using the code sample below.
             * Get the result (succeed or failed) in completion block.
             * If session is no longer valid , call the logout API which remove Access token stored in keychain. Again call authenticate:fromViewController:withCompletionHandler: API to get Access token.
             */
            
            // Initilize the PCAPISDK and call the API.
            
            PCAPI *thumbnailAPI = [[PCAPISDK sharedInstance] downloadThumbnail:params withCompletionHandler:^(PCAPIQueryResponse *response) {
                // Get the result in this block.
                NSData *fileData = [NSData dataWithContentsOfURL:response.downloadedURL];
                if (fileData) {
                    [fileData writeToFile:[blocksafeSelf thumbnailCachePath] atomically:YES];
                }
                
                
                if ([imageView.imageID isEqualToString:blocksafeSelf.photoReference]) { // cell is still representing same resource
                    
                    
                    if (fileData) {
                        
                        UIImage *thumbImage = [UIImage imageWithContentsOfFile:[blocksafeSelf thumbnailCachePath]];
                        if (thumbImage) {
                            
                            imageView.image = thumbImage;
                            completion(YES);
                        } else {
                            
                            imageView.image = [blocksafeSelf defaultPic];
                            completion(YES);
                        }
                        
                    } else {
                        
                        imageView.image = [blocksafeSelf defaultPic];
                        completion(YES);
                    }
                } else {
                
                    completion(NO);
                }
            }];
            
            imageView.api = thumbnailAPI;
        }
    } else {
        
        imageView.image = [self defaultPic];
        completion(YES);
    }
}

- (NSDateComponents *)convertServerDateStringToNSDateComponents:(NSString *)dateString {

    // Convert string to date object
    NSDateFormatter* serverFormat = [[NSDateFormatter alloc] init];
    [serverFormat setDateFormat:@"yyyy-MM-dd"];
    NSDate *date = [serverFormat dateFromString:dateString];
    NSCalendar *gregorian = [[NSCalendar alloc]
                             initWithCalendarIdentifier:NSCalendarIdentifierGregorian];
    NSDateComponents *dateComponent =
    [gregorian components:(kCFCalendarUnitYear | kCFCalendarUnitMonth |kCFCalendarUnitDay) fromDate:date];
    return dateComponent;
}

@end
