//Copyright © 2015-2017, Verizon and/or its Licensors. All rights reserved.

#import "PCShareResourceDownloader.h"

static NSURL *documentsDirectoryURL;

@interface PCShareResourceDownloader () <NSURLSessionDownloadDelegate>
@property (nonatomic, strong) NSURLSessionDownloadTask *downloadTask;
@property (nonatomic, strong) NSURL *downloadedURL;
@property (nonatomic, copy) void (^completionHandler)(NSURLResponse *, NSURL *, NSError*);
@property (nonatomic, copy) void (^progressHandler)(long long, long long, long long);

@end

@implementation PCShareResourceDownloader

-(NSURLSession *) setBackgroundSession {
    __block NSURLSessionConfiguration* defaultSessionConfig;
    static dispatch_once_t onceToken;
    dispatch_once(&onceToken, ^{
         defaultSessionConfig = [NSURLSessionConfiguration backgroundSessionConfigurationWithIdentifier:@"HubbleMediaBrowser"];
    });
     return [NSURLSession sessionWithConfiguration:defaultSessionConfig
                                                       delegate:self
                                                  delegateQueue:nil];
}

+ (NSURL *)documentsDirectoryURL {
    if (!documentsDirectoryURL) {
        NSArray *URLs = [[NSFileManager defaultManager] URLsForDirectory:NSDocumentDirectory inDomains:NSUserDomainMask];
        documentsDirectoryURL = [URLs objectAtIndex:0];
    }
    return documentsDirectoryURL;
}

- (void)downloadSharedResource:(NSString *)url progress:(void(^)(long long bytesWritten, long long totalBytesWritten, long long totalBytesExpectedToWrite))progress withCompletionHandler:(void(^)(NSURLResponse *response, NSURL *location, NSError *error))completion {
    self.completionHandler  = completion;
    self.progressHandler    = progress;
    self.downloadTask = [[self setBackgroundSession] downloadTaskWithURL:[NSURL URLWithString:url]];
    [self.downloadTask resume];
}

- (void)cancelAPI {
    [self.downloadTask cancel];
    self.downloadTask = nil;
    self.completionHandler = nil;
    self.progressHandler = nil;
}

#pragma mark -
#pragma mark NSURLSessionDownloadDelegate

- (void)URLSession:(NSURLSession *)session task:(NSURLSessionTask *)task
   didSendBodyData:(int64_t)bytesSent
    totalBytesSent:(int64_t)totalBytesSent
totalBytesExpectedToSend:(int64_t)totalBytesExpectedToSend {
    if (self.progressHandler) {
        self.progressHandler(bytesSent, totalBytesSent, totalBytesExpectedToSend);
    }
}

- (void)URLSession:(NSURLSession *)session downloadTask:(NSURLSessionDownloadTask *)downloadTask
didFinishDownloadingToURL:(NSURL *)location {
    NSError *error;
    NSFileManager *fileManager = [NSFileManager defaultManager];
    NSString *destinationFilename = downloadTask.originalRequest.URL.lastPathComponent;
    
    NSURL *destinationURL = [[PCShareResourceDownloader documentsDirectoryURL] URLByAppendingPathComponent:destinationFilename];
    
    if ([fileManager fileExistsAtPath:[destinationURL path]]) {
        [fileManager removeItemAtURL:destinationURL error:nil];
    }
    
    BOOL success = [fileManager copyItemAtURL:location
                                        toURL:destinationURL
                                        error:&error];
    if (success) {
        self.downloadedURL = destinationURL;
    }
}
- (void)URLSession:(NSURLSession *)session task:(NSURLSessionTask *)task
didCompleteWithError:(nullable NSError *)error {
    if (self.completionHandler) {
        self.completionHandler(task.response,_downloadedURL,error);
        self.completionHandler = nil;
    }
    
    if (self.progressHandler) {
        self.progressHandler = nil;
    }
}

- (void)URLSession:(NSURLSession *)session downloadTask:(NSURLSessionDownloadTask *)downloadTask
      didWriteData:(int64_t)bytesWritten
 totalBytesWritten:(int64_t)totalBytesWritten
totalBytesExpectedToWrite:(int64_t)totalBytesExpectedToWrite {
    if (self.progressHandler) {
        self.progressHandler(bytesWritten, totalBytesWritten, totalBytesExpectedToWrite);
    }
}

@end
