//Copyright © 2015-2017, Verizon and/or its Licensors. All rights reserved.

#import "PlaylistViewController.h"
#import "UIViewController+Dismiss.h"
#import "PlaylistItemViewController.h"
#import "UIViewController+Dismiss.h"
#import "UIViewController+Alert.h"
#import "UIViewController+RightBarButtonAction.h"
#import "UIViewController+PlaylistActions.h"
#import "UIViewController+Spinner.h"
#import "CustomTableViewCell.h"
#import "PCAccountInfo+additions.h"
#import "NSArray+Addition.h"
#import "MoveToPlaylistViewController.h"

@interface BaseTableViewController (Private)

@property (nonatomic, strong) NSString *emptyMessage;

@end

@interface PlaylistViewController () <UITableViewDataSource, CustomTableViewCellDelegate>

@property (nonatomic, strong) NSArray *playlists;

@end

@implementation PlaylistViewController

- (void)viewDidLoad {

    [super viewDidLoad];
        
    [self addCancelButton];
    
    self.refreshControl = [[UIRefreshControl alloc] init];
    [self.refreshControl addTarget:self action:@selector(reloadData) forControlEvents:UIControlEventValueChanged];
    [self.tableView addSubview:self.refreshControl];

    [self.refreshControl beginRefreshing];
    
    
    
    switch (self.playlistScreenType) {
        case playlistScreenTypeMusic: {
        
            if ([self isKindOfClass:[MoveToPlaylistViewController class]]) {
                
                [self addRightBarButtonItemWithActions:[NSNumber numberWithInt:createPlaylistRightBarAction], nil];

            } else {
            
                [self addRightBarButtonItemWithActions:[NSNumber numberWithInt:accountInfoRightBarAction], [NSNumber numberWithInt:searchRightBarAction], [NSNumber numberWithInt:createPlaylistRightBarAction], [NSNumber numberWithInt:signOutRightBarAction], nil];
            }
            
            
            self.navigationItem.title = NSLocalizedString(@"Playlists", nil);
            
            self.emptyMessage = NSLocalizedString(@"Playlist is empty", nil);
        }
            break;
        case playlistScreenTypeAlbum: {
        
            if ([self isKindOfClass:[MoveToPlaylistViewController class]]) {
                
                [self addRightBarButtonItemWithActions:[NSNumber numberWithInt:createAlbumRightBarAction], nil];
                
            } else {
                
                [self addRightBarButtonItemWithActions:[NSNumber numberWithInt:accountInfoRightBarAction], [NSNumber numberWithInt:searchRightBarAction], [NSNumber numberWithInt:createAlbumRightBarAction], [NSNumber numberWithInt:signOutRightBarAction], nil];
            }

            
            self.navigationItem.title = NSLocalizedString(@"Albums", nil);
            
            self.emptyMessage = NSLocalizedString(@"Album is empty", nil);

        }
            break;
        default:
            break;
    }

    [self reloadData];
    
}

- (void)reloadData {
    
    __weak PlaylistViewController *blocksafeSelf = self;
    switch (self.playlistScreenType) {
        case playlistScreenTypeMusic:{
            /*
             @Description :
             This API is used to fetch all music playlist in an account. Once the fetch process is succeed/failed result will come with completion block.
             
             @Parameters: nil
             * For providing access token to SDK user have to logged In.
             
             @CompletionHandler :
             This completion block gets called once fetch process is succeed/failed. completion block contains below parameter.
             PCAPIQueryResponse :
             * Stores all the server response and error for this API.
             NSArray :
             * An array of PCPlaylist object. PCPlaylist class stores all the details of a Playlist.
             * Using the PCPlayList object we can get all file using getFileWithCompletionHandler method. This method fetch all PCAudio file related to this PCPlaylist object.
             
             @To fetch all music playlist in an account, you must take the following steps:
             * Include VerizonCloudAPI.framework header file in your class.
             #import <VerizonCloudAPI/VerizonCloudAPI.h>
             * Initalize the PCAPISDK object and call the getAudioPlaylistsWithCompletionHandler: API.
             * Perform the getAudioPlaylistsWithCompletionHandler: API call using the code sample below.
             * Get the result (succeed or failed) in completion block.
             * If session is no longer valid , call the logout API which remove Access token stored in keychain. Again call authenticate:fromViewController:withCompletionHandler: API to get Access token.
             */
            
            // Sample code for getAudioPlaylistsWithCompletionHandler: API
            [[PCAPISDK sharedInstance] getAudioPlaylistsWithCompletionHandler:^(NSArray *itemList, PCAPIQueryResponse *response) {
                [blocksafeSelf.refreshControl endRefreshing];
                
                switch (response.urlResponse.statusCode) {
                    case PCAPIResponseCodeOK: {
                        
                        blocksafeSelf.playlists = itemList;
                        [blocksafeSelf refreshView];
                    }
                        break;
                    case PCAPIResponseCodeUnauthorized:
                        [blocksafeSelf showUnAuthorizedAlert];
                        break;
                    case PCAPIResponseCodeServiceUnavailable:
                        [blocksafeSelf showServerUnavailableAlert];
                        break;
                    default:
                        [blocksafeSelf showOperationFailedAlert:NSLocalizedString(@"Playlist fetch", nil)];
                        break;
                }
                
            }];
        }
            break;
        case playlistScreenTypeAlbum:{
            /*
             @Description :
             This API is used to fetch all image, image-video and video albums in an account. Once the fetch process is succeed/failed result will come with completion block.
             
             @Parameters: nil
             * For providing access token to SDK user have to logged In.
             
             @CompletionHandler :
             This completion block gets called once fetch process is succeed/failed. completion block contains below parameter.
             PCAPIQueryResponse :
             * Stores all the server response and error for this API.
             NSArray :
             * An array of PCAlbum object. PCAlbum class stores information of image, video and image-video related album.
             * Using the PCAlbum object we can get all file using getFileWithCompletionHandler method. This method fetch all file related to this PCAlbum object.
             
             @To fetch all image, image-video and video albums in an account, you must take the following steps:
             * Include VerizonCloudAPI.framework header file in your class.
             #import <VerizonCloudAPI/VerizonCloudAPI.h>
             * Initalize the PCAPISDK object and call the getPhotoAlbumsWithCompletionHandler: API.
             * Perform the getPhotoAlbumsWithCompletionHandler: API call using the code sample below.
             * Get the result (succeed or failed) in completion block.
             * If session is no longer valid , call the logout API which remove Access token stored in keychain. Again call authenticate:fromViewController:withCompletionHandler: API to get Access token.
             */
            
            // Sample code for getPhotoAlbumsWithCompletionHandler: API
            [[PCAPISDK sharedInstance] getPhotoAlbumsWithCompletionHandler:^(NSArray *itemList, PCAPIQueryResponse *response) {
                [blocksafeSelf.refreshControl endRefreshing];
                switch (response.urlResponse.statusCode) {
                    case PCAPIResponseCodeOK:{
                        
                        blocksafeSelf.playlists = itemList;
                        [blocksafeSelf refreshView];
                    }
                        break;
                    case PCAPIResponseCodeUnauthorized:
                        [blocksafeSelf showUnAuthorizedAlert];
                        break;
                    case PCAPIResponseCodeServiceUnavailable:
                        [blocksafeSelf showServerUnavailableAlert];
                        break;
                    default:
                        [blocksafeSelf showOperationFailedAlert:NSLocalizedString(@"Album fetch", nil)];
                        break;
                }
            }];
        }
            break;
        case playlistScreenTypeUnknown:
            break;
    }
}

- (void)refreshView {

    if (self.playlists.count) {
        [self.tableView reloadData];
    } else {
        [self showOKAlertWithTitle:NSLocalizedString(@"Empty", nil)
                    andMessage:self.emptyMessage attributedString:nil okAction:nil];
    }
}

#pragma mark -
#pragma mark Playlist Action callback

- (void)didBeginDoingPlaylistAction:(PlaylistAction)action {

    [self showSpinner];
}

- (void)didFinishDoingAction:(PlaylistAction)action onPlaylist:(id)playlist result:(id)result response:(id)response {

    
    PCAPIQueryResponse *APIResponse = (PCAPIQueryResponse *)response;
    
    [self hideSpinner];
    
    
    switch (APIResponse.urlResponse.statusCode) {
        case PCAPIResponseCodeResourceCreated:
        case PCAPIResponseCodeOK: {
            
            switch (action) {
                case deletePlaylistAction: {
                
                    if (playlist) {
                        NSMutableArray *playlists = [NSMutableArray arrayWithArray:self.playlists];
                        [playlists removeObject:playlist];
                        self.playlists = [NSArray arrayWithArray:playlists];
                        [self.tableView reloadData];
                    }
                }
                    break;
                case renamePlaylistAction: {
                    
                    if (playlist) {
                                                
                        self.playlists = [self.playlists arrayByReplacingObject:playlist
                                                                     withObject:result
                                                                  andSortByKeyPath:@"type"
                                                                      ascending:YES];
                        [self.tableView reloadData];
                    }
                }
                    break;
                case createSharePlaylistAction : {
                    [self showOKAlertWithTitle:NSLocalizedString(@"Success", nil) andMessage:NSLocalizedString(@"Share created successfully", nil) attributedString:nil okAction:nil];
                }
                default:
                    break;
            }
        }
            break;
        case PCAPIResponseCodeUnauthorized:
            [self showUnAuthorizedAlert];
            break;
        case PCAPIResponseCodeServiceUnavailable:
            [self showServerUnavailableAlert];
            break;
        default:
            [self showOperationFailedAlert:[self stringForPlaylistAction:action]];
            break;
    }
}

#pragma mark -
#pragma mark UITableView methods

- (NSInteger)tableView:(UITableView *)tableView numberOfRowsInSection:(NSInteger)section {
    return self.playlists.count;
}

- (UITableViewCell *)tableView:(UITableView *)tableView cellForRowAtIndexPath:(NSIndexPath *)indexPath {
    CustomTableViewCell *cell = (CustomTableViewCell *)[tableView dequeueReusableCellWithIdentifier:@"playlistCell" forIndexPath:indexPath];
    
    PCPlaylist *playlist = [self.playlists objectAtIndex:indexPath.row];
    
    cell.textLabel.text = playlist.name;
    cell.delegate = self;
    
    return cell;
}

- (void)tableView:(UITableView *)tableView didSelectRowAtIndexPath:(NSIndexPath *)indexPath {
    
    PCPlaylist *playlist = [self.playlists objectAtIndex:indexPath.row];
    
    UIStoryboard *storyBoard = [UIStoryboard storyboardWithName:@"Main" bundle:nil];

    PlaylistItemViewController *controller = (PlaylistItemViewController *)[storyBoard instantiateViewControllerWithIdentifier: @"PlaylistItemViewController"];
    controller.playlist = playlist;
    [self.navigationController pushViewController:controller animated:YES];
}

- (BOOL)tableView:(UITableView *)tableView canEditRowAtIndexPath:(NSIndexPath *)indexPath {
    return YES;
}

- (UITableViewCellEditingStyle)tableView:(UITableView *)tableView editingStyleForRowAtIndexPath:(NSIndexPath *)indexPath {
    return UITableViewCellEditingStyleDelete;
}

- (void)tableView:(UITableView *)tableView commitEditingStyle:(UITableViewCellEditingStyle)editingStyle forRowAtIndexPath:(NSIndexPath *)indexPath {
    if (editingStyle == UITableViewCellEditingStyleDelete) {
        
        PCPlaylist *playlist = [self.playlists objectAtIndex:indexPath.row];
        [self executeAction:deletePlaylistAction forPlaylist:playlist];
    }
}

#pragma mark -
#pragma mark CustomTableViewCell delegate

- (void)tableViewCellDidLongPressed:(UITableViewCell *)cell {
    DLog(@"Table view cell did long pressed");
    
    NSIndexPath *indexPath = [self.tableView indexPathForCell:cell];
    PCPlaylist *selectedPlaylist = [self.playlists objectAtIndex:indexPath.row];
    [self showActionsForSelectedPlaylist:selectedPlaylist];
}

#pragma mark -
#pragma mark Playlist 

- (void)didBeginDoingRightBarButtonAction:(rightBarAction)action {
    [self showSpinner];
}

#pragma mark -
#pragma mark Right bar actions
- (void)didFinishDoingRightBarButtonAction:(rightBarAction)action :(id)result :(id)response {
    
    [self hideSpinner];
    
    PCAPIQueryResponse *APIResponse = (PCAPIQueryResponse *)response;
    
    switch (APIResponse.urlResponse.statusCode) {
            
        case PCAPIResponseCodeResourceCreated:
        case PCAPIResponseCodeOK: {
            
            switch (action) {
                case accountInfoRightBarAction: {
                    
                    PCAccountInfo *accountInfo = (PCAccountInfo *)result;
                    [self showOKAlertWithTitle:[self stringForRightBarAction:action]
                                    andMessage:[accountInfo humanReadableAccountInfo]
                              attributedString:nil
                                      okAction:nil];
                }
                    break;
                case refreshAccountBarAction: {
                    
                    [self showOKAlertWithTitle:NSLocalizedString(@"Success", nil) andMessage:NSLocalizedString(@"Successfully refreshed account. Please pull to refresh the screen", nil) attributedString:nil okAction:nil];
                }
                    break;
                case createPlaylistRightBarAction: {
                
                    if (result && [result isKindOfClass:[PCPlaylist class]]) {
                        
                        self.playlists = [self.playlists arrayByAddingObject:result andSortByKeyPath:@"type" ascending:YES];
                    }
                    [self.tableView reloadData];
                }
                    break;
                default:
                    break;
            }
            
        }
            break;
        case PCAPIResponseCodeUnauthorized:
            [self showUnAuthorizedAlert];
            break;
        case PCAPIResponseCodeServiceUnavailable:
            [self showServerUnavailableAlert];
            break;
        default:
            [self showOperationFailedAlert:[self stringForRightBarAction:action]];
            break;
    }    
}


- (id)dataSourceForRightBarButtonAction:(rightBarAction)action {
    switch (self.playlistScreenType) {
        case playlistScreenTypeMusic:
            return @"music";
            break;
        case playlistScreenTypeAlbum:
            return @"image-video";
            break;
        default:
            return nil;
            break;
    }
}

@end
