//Copyright © 2015-2017, Verizon and/or its Licensors. All rights reserved.

#import <XCTest/XCTest.h>
#import "PCTestConstants.h"

@interface PCCreateContactAPITests : XCTestCase

@end

@implementation PCCreateContactAPITests

- (void)setUp {
    [super setUp];
    // Put setup code here. This method is called before the invocation of each test method in the class.
}

- (void)tearDown {
    // Put teardown code here. This method is called after the invocation of each test method in the class.
    [super tearDown];
}

/*!
 * Test Case: auth token is a required parameter. SDK should throw an exception in case auth token is nil.
 */
- (void)testNilAuthToken {
    
    // If User is not logged in, create Contact api should throw an exception
    if (![[PCAPISDK sharedInstance] isUserLoggedIn]) {
        
        PCCreateContactAPIParams *params = [[PCCreateContactAPIParams alloc] init];
        
        XCTAssertThrows([[PCAPISDK sharedInstance] createContact:params withCompletionHandler:^(PCContact *contact, PCAPIQueryResponse *response) {}],@"create Contact API with nil access token");
    }
    
}

/*!
 * Test Case: If response code is 200, create contact API will create provided contact. In all other cases (error),it will not create contact.
 */

- (void)testToCreateContactWithValues {
    
    if ([[PCAPISDK sharedInstance] isUserLoggedIn]) {
        
        PCCreateContactAPIParams *params = [[PCCreateContactAPIParams alloc] init];
        params.firstName = @"test_firstname";
        params.lastName = @"test_lastname";
        params.middleName = @"test_middlename";
        
        XCTestExpectation *expectation = [self expectationWithDescription:@"Create contact API Result"];
        [[PCAPISDK sharedInstance] createContact:params withCompletionHandler:^(PCContact *contact, PCAPIQueryResponse *response) {
            switch (response.urlResponse.statusCode) {
                case PCAPIResponseCodeResourceCreated:
                    XCTAssertTrue([contact isMemberOfClass:[PCContact class]],@"create contact API return PCContact object");
                    break;
                default:
                    XCTFail(@"Create contact API failed");
                    break;
            }
            [expectation fulfill];
        }];
        [self waitForExpectationsWithTimeout:ExpectationTimeOut handler:^(NSError *error) {
            
            if (error) {
                NSLog(@"Timeout Error: %@", error);
            }
            
        }];
        
    } else {
        XCTFail(@"Test failed due to nil access token");
    }
}

- (void)testToCreateContactWithoutValues {
    
    if ([[PCAPISDK sharedInstance] isUserLoggedIn]) {
        
        PCCreateContactAPIParams *params = [[PCCreateContactAPIParams alloc] init];
        
        XCTestExpectation *expectation = [self expectationWithDescription:@"Create contact API Result"];
        [[PCAPISDK sharedInstance] createContact:params withCompletionHandler:^(PCContact *contact, PCAPIQueryResponse *response) {
              XCTAssertFalse([contact isMemberOfClass:[PCContact class]],@"create contact API fail to return PCContact object");
            [expectation fulfill];
        }];
        [self waitForExpectationsWithTimeout:ExpectationTimeOut handler:^(NSError *error) {
            
            if (error) {
                NSLog(@"Timeout Error: %@", error);
            }
            
        }];
        
    } else {
        XCTFail(@"Test failed due to nil access token");
    }
}

@end
