//Copyright © 2015-2017, Verizon and/or its Licensors. All rights reserved.

#import <XCTest/XCTest.h>
#import "PCTestConstants.h"

@interface PCDeletePlaylistAPITests : XCTestCase

@end

@implementation PCDeletePlaylistAPITests

- (void)setUp {
    [super setUp];
    // Put setup code here. This method is called before the invocation of each test method in the class.
}

- (void)tearDown {
    // Put teardown code here. This method is called after the invocation of each test method in the class.
    [super tearDown];
}

/*!
 * Test Case: auth token is a required parameter. SDK should throw an exception in case auth token is nil.
 */
- (void)testNilAuthToken {
    
    // If User is not logged in, Delete Playlist api should throw an exception
    if (![[PCAPISDK sharedInstance] isUserLoggedIn]) {
        
        PCDeletePlaylistAPIParams *params = [[PCDeletePlaylistAPIParams alloc] init];
        XCTAssertThrows([[PCAPISDK sharedInstance] deletePlaylist:params withCompletionHandler:^(PCAPIQueryResponse *response) {}],@"Delete playlist API with nil access token");
    }
    
}

/*!
 * Test Case:PlaylistID is required to delete playlist response,otherwise test will fail.
 */

- (void)testForRequiredPlaylistID {
    
    if ([[PCAPISDK sharedInstance] isUserLoggedIn]) {
        
        PCDeletePlaylistAPIParams* params = [[PCDeletePlaylistAPIParams alloc] init];
        XCTAssertThrows([[PCAPISDK sharedInstance] deletePlaylist:params withCompletionHandler:^(PCAPIQueryResponse *response) {}],@"Delete playlist API required playlist ID");
        
    } else {
        XCTFail(@"Test failed due to nil access token");
    }
}

/*!
 * Test Case: If response code is 200, delete playlist API delete provided playlist. In all other cases (error),it will not delete playlist.
 */

- (void)testToDeletePlaylist {
    
    if ([[PCAPISDK sharedInstance] isUserLoggedIn]) {
        
        XCTestExpectation *expectation = [self expectationWithDescription:@"Delete playlist API result"];
        
        PCGetPlaylistAPIParams *params = [[PCGetPlaylistAPIParams alloc] init];
        
        [[PCAPISDK sharedInstance] getPlaylist:params withCompletionHandler:^(NSArray *playlist, PCAPIQueryResponse *response) {
            
            switch (response.urlResponse.statusCode) {
                    
                case PCAPIResponseCodeOK: {
                    
                    XCTAssertGreaterThanOrEqual([playlist count], 0);
                    
                    if (playlist.count) {
                        
                        PCPlaylist *object = [playlist objectAtIndex:0];
                        
                        PCDeletePlaylistAPIParams *params = [[PCDeletePlaylistAPIParams alloc] init];
                        params.playlistUID = object.UID;
                        
                        [[PCAPISDK sharedInstance] deletePlaylist:params withCompletionHandler:^(PCAPIQueryResponse *response) {
                            
                            switch (response.urlResponse.statusCode) {
                                case PCAPIResponseCodeOK:
                                    XCTAssertTrue([playlist isMemberOfClass:[PCPlaylist class]],@"Delete playlist definition API return PCPlaylist object");
                                    break;
                                default:
                                    XCTFail(@"Delete playlist API failed");
                                    break;
                            }
                            [expectation fulfill];
                        }];
                    }
                    
                }
                    break;
                    
                default:
                    XCTFail(@"Test Failed due to absence of playlist");
                    break;
            }
            [expectation fulfill];
        }];
        [self waitForExpectationsWithTimeout:ExpectationTimeOut*2 handler:^(NSError *error) {
            
            if (error) {
                NSLog(@"Timeout Error: %@", error);
            }
            
        }];
    } else {
        XCTFail(@"Test failed due to nil access token");
    }
}

@end
