//Copyright © 2015-2017, Verizon and/or its Licensors. All rights reserved.

#import <XCTest/XCTest.h>
#import "PCTestConstants.h"

@interface PCUpdateContactAPITests : XCTestCase

@end

@implementation PCUpdateContactAPITests

- (void)setUp {
    [super setUp];
    // Put setup code here. This method is called before the invocation of each test method in the class.
}

- (void)tearDown {
    // Put teardown code here. This method is called after the invocation of each test method in the class.
    [super tearDown];
}

/*!
 * Test Case: auth token is a required parameter. SDK should throw an exception in case auth token is nil.
 */
- (void)testNilAuthToken {
    
    // If User is not logged in, Update Contact api should throw an exception.
    if (![[PCAPISDK sharedInstance] isUserLoggedIn]) {
        
        PCUpdateContactAPIParams *params = [[PCUpdateContactAPIParams alloc] init];
        
        XCTAssertThrows([[PCAPISDK sharedInstance] updateContact:params withCompletionHandler:^(PCContact *contact, PCAPIQueryResponse *response) {}] ,@"Update Contact API with nil access token");
    }
    
}

/*!
 * Test Case:contactId is required to update Contact,otherwise test will fail.
 */

- (void)testForRequiredContactID {
    
    if ([[PCAPISDK sharedInstance] isUserLoggedIn]) {
        
        PCUpdateContactAPIParams *params = [[PCUpdateContactAPIParams alloc] init];
        XCTAssertThrows([[PCAPISDK sharedInstance] updateContact:params withCompletionHandler:^(PCContact *contact, PCAPIQueryResponse *response) {}] ,@"Update Contact API required contact ID");
    }else {
        XCTFail(@"Test failed due to nil access token");
    }
}

/*!
 * Test version is required to update Contact,otherwise test will fail.
 */

- (void)testForRequiredVersion {
    
    if ([[PCAPISDK sharedInstance] isUserLoggedIn]) {
        
        PCContactAPIParams *params = [[PCContactAPIParams alloc] init];
        
        [[PCAPISDK sharedInstance] fetchContacts:params withCompletionHandler:^(NSArray *contacts, PCAPIQueryResponse *response) {
            
            if (contacts.count) {
                PCContact *contact = [contacts objectAtIndex:0];
                
                PCUpdateContactAPIParams *params = [[PCUpdateContactAPIParams alloc] init];
                params.contactID = contact.ID;
                
                XCTAssertThrows([[PCAPISDK sharedInstance] updateContact:params withCompletionHandler:^(PCContact *contact, PCAPIQueryResponse *response) {}] ,@"Update Contact API required version");
            }
        }];
    }else {
        XCTFail(@"Test failed due to nil access token");
    }
}

/*!
 * Test Case: If response code is 200, update contact API will update provided contact. In all other cases (error),it will not update contact.
 */

- (void)testToUpdateContact {
    
    if ([[PCAPISDK sharedInstance] isUserLoggedIn]) {
        
        XCTestExpectation *expectation = [self expectationWithDescription:@"Fetch Contact result"];
        
        PCContactAPIParams *params = [[PCContactAPIParams alloc] init];
        
        [[PCAPISDK sharedInstance] fetchContacts:params withCompletionHandler:^(NSArray *contacts, PCAPIQueryResponse *response) {
            switch (response.urlResponse.statusCode) {
                    
                case PCAPIResponseCodeOK: {
                    
                    XCTAssertGreaterThanOrEqual([contacts count], 0);
                    
                    if (contacts.count) {
                        
                        PCContact *object = [contacts objectAtIndex:0];
                        
                        PCUpdateContactAPIParams *params = [[PCUpdateContactAPIParams alloc] initWithPCContact:object];
                        params.contactID = object.ID;
                        params.firstName = @"Test Name";
                        
                        [[PCAPISDK sharedInstance] updateContact:params withCompletionHandler:^(PCContact *contact, PCAPIQueryResponse *response) {
                            switch (response.urlResponse.statusCode) {
                                case PCAPIResponseCodeOK:
                                    XCTAssertTrue([contact isMemberOfClass:[PCContact class]],@"Update contact API return PCContact object");
                                    break;
                                default:
                                    XCTFail(@"Update contact API failed");
                                    break;
                            }
                            [expectation fulfill];
                        }];
                    }
                    
                }
                    break;
                    
                default:
                    XCTFail(@"Test Failed due to absence of contacts");
                    break;
            }
            [expectation fulfill];
        }];
        
        [self waitForExpectationsWithTimeout:ExpectationTimeOut*2 handler:^(NSError *error) {
            
            if (error) {
                NSLog(@"Timeout Error: %@", error);
            }
            
        }];
    } else {
        XCTFail(@"Test failed due to nil access token");
    }
}



@end
