//Copyright © 2015-2017, Verizon and/or its Licensors. All rights reserved.

#import "NSArray+Addition.h"

@implementation NSArray (Addition)

- (NSArray *)arrayByReplacingObject:(id)object1 withObject:(id)object2 andSortByKeyPath:(NSString *)sortKeyPath ascending:(BOOL)ascending {
    
    NSMutableArray *result = [NSMutableArray arrayWithArray:self];
    
    if (object1 && object2) {
        
        NSUInteger indexOfTheReplaceObject = [self indexOfObject:object1];
        
        if (indexOfTheReplaceObject != NSNotFound) {
            [result replaceObjectAtIndex:indexOfTheReplaceObject withObject:object2];
        }
    }
    
    if (sortKeyPath.length) {
        return [result sortedArrayByKeyPath:sortKeyPath ascending:ascending] ;
    }
    return [NSArray arrayWithArray:result];
}

- (NSArray *)arrayByAddingObject:(id)obj andSortByKeyPath:(NSString *)sortKeyPath ascending:(BOOL)ascending {
    
    NSArray *result = [NSArray arrayWithArray:self];
    
    if (obj) {
        
        result = [result arrayByAddingObject:obj];
    }
    
    if (sortKeyPath.length) {
        
        return [result sortedArrayByKeyPath:sortKeyPath ascending:ascending] ;
    }
    return [NSArray arrayWithArray:result];
}

- (NSArray *)sortedArrayByKeyPath:(NSString *)keyPath ascending:(BOOL)ascending {
    
    NSArray *result;
    
    if (keyPath.length) {
        
        NSSortDescriptor* sortDescriptor = [NSSortDescriptor sortDescriptorWithKey:keyPath ascending:ascending selector:@selector(caseInsensitiveCompare:)];
        result = [self sortedArrayUsingDescriptors:[NSArray arrayWithObject:sortDescriptor]];
    }
    return result;
}

- (NSDictionary *)groupUsingBlock:(id<NSCopying> (^)(id obj))block {
    NSMutableDictionary *dictionary = [NSMutableDictionary dictionary];
    
    for (id obj in self) {
        id<NSCopying> key = block(obj);
        
        if (!key) {
            continue;
        }
        
        if (!dictionary[key]) {
            
            NSMutableArray *arr = [NSMutableArray array];
            dictionary[key] = arr;
        }
        
        [dictionary[key] addObject:obj];
    }
    return [dictionary copy];
}


- (NSDictionary *)groupByKeyPath:(NSString *)keyPath {
    return [self groupUsingBlock:^id<NSCopying>(id obj) {
        return [obj valueForKeyPath:keyPath];
    }];
}

@end
