//Copyright © 2015-2017, Verizon and/or its Licensors. All rights reserved.

#import "FavoritesViewController.h"
#import "UIViewController+Spinner.h"
#import "UIViewController+Alert.h"
#import "UIViewController+RightBarButtonAction.h"

@interface FileGridViewController (Protected)

@property (nonatomic, strong) NSString *emptyViewMessage;
@property (nonatomic, retain) UIRefreshControl *refreshControl;

- (void)refreshView;
- (void)alertController:(UIAlertController *)alertController
     addActionWithTitle:(NSString *)title
                  style:(UIAlertActionStyle)style
              andAction:(void (^)(UIAlertAction * _Nonnull action))handler;

@end

@interface FavoritesViewController ()

@end

@implementation FavoritesViewController

- (void)reloadData {
    
    /*
     @Description -
     This API is used to fetch the list of favorite files or folders under the specified virtual folder. Once the fetch process is succeed/failed result will come with completion block.
     
     @Parameters -
     PCGetFavoritesAPIParams:
     * This class stores all the parameters required to fetch the list of favorite files or folders under the specified virtual folder.
     *  For providing access token to SDK user have to logged In.
     * If user will not provide required parameter, API will throw an NSInvalidArgumentException and user have to handle it.
     * FavoriteResourceType is optional parameter to call this API.
     * More information about parameters you will find in PCGetFavoritesAPIParams.h class.
     
     @CompletionHandler :
     This completion block gets called once the fetch process is succeed/failed. completion block contains below parameter.
     PCAPIQueryResponse :
     * Stores all the server response and error for this API.
     NSArray :
     * An array of PCMetadata object.
     
     @To fetch fetch the list of favorite files or folders under the specified virtual folder, you must take the following steps:
     * Include VerizonCloudAPI.framework header file in your class.
     #import <VerizonCloudAPI/VerizonCloudAPI.h>
     * Initalize the PCGetFavoritesAPIParams class and provide the required parameter.
     * Initalize the PCAPISDK object and call the getFavorites:withCompletionHandler: API.
     * Perform the getFavorites:withCompletionHandler: API call using the code sample below.
     * Get the result (succeed or failed) in completion block.
     * If session is no longer valid , call the logout API which remove Access token stored in keychain. Again call authenticate:fromViewController:withCompletionHandler: API to get Access token.
     */
    
    PCGetFavoritesAPIParams *params = [[PCGetFavoritesAPIParams alloc] init];
    
    __weak FavoritesViewController *blocksafeSelf = self;
    [[PCAPISDK sharedInstance] getFavorites:params withCompletionHandler:^(NSArray *favoritesItems, PCAPIQueryResponse *response) {
        [blocksafeSelf hideSpinner];
        [blocksafeSelf.refreshControl endRefreshing];
        
        switch (response.urlResponse.statusCode) {
            case PCAPIResponseCodeOK: {
                blocksafeSelf.dataSource = [NSArray arrayWithArray:favoritesItems];
                [blocksafeSelf refreshView];
            }
                break;
            case PCAPIResponseCodeUnauthorized:
                [self showUnAuthorizedAlert];
                break;
            case PCAPIResponseCodeServiceUnavailable:
                [self showServerUnavailableAlert];
                break;
            default:
                [self showOperationFailedAlert:NSLocalizedString(@"Trash fetch", nil)];
                break;
        }
        
    }];
}

- (void)viewDidLoad {
    
    [super viewDidLoad];
    
    self.title = NSLocalizedString(@"Favorites", nil);
    
    self.emptyViewMessage = NSLocalizedString(@"No items found in Favorites", nil);
    
    [self showSpinner];
    [self.refreshControl beginRefreshing];
    
    [self reloadData];
    // Do any additional setup after loading the view.
}

- (id)dataSourceForRightBarButtonAction:(rightBarAction)action {
    return nil;
}

#pragma mark -
#pragma mark RightBarButtonAction callback

- (void)didBeginDoingRightBarButtonAction:(rightBarAction)action {
    
    [self showSpinner];
}

- (void)didFinishDoingRightBarButtonAction:(rightBarAction)action :(id)result :(id)response {
    
    [self hideSpinner];
    
    PCAPIQueryResponse *APIResponse = (PCAPIQueryResponse *)response;
    
    switch (APIResponse.urlResponse.statusCode) {
        case PCAPIResponseCodeOK: {
            
            switch (action) {
                case photoFavoriteRightBarAction: {
                    self.dataSource = (NSArray *)result;
                    [self refreshView];
                }
                    break;
                default:
                    break;
            }
            
        }
            break;
        case PCAPIResponseCodeUnauthorized:
            [self showUnAuthorizedAlert];
            break;
        case PCAPIResponseCodeServiceUnavailable:
            [self showServerUnavailableAlert];
            break;
        default:
            [self showOperationFailedAlert:[self stringForRightBarAction:action]];
            break;
    }
}

- (void)didReceiveMemoryWarning {
    [super didReceiveMemoryWarning];
    // Dispose of any resources that can be recreated.
}

/*
 #pragma mark - Navigation
 
 // In a storyboard-based application, you will often want to do a little preparation before navigation
 - (void)prepareForSegue:(UIStoryboardSegue *)segue sender:(id)sender {
 // Get the new view controller using [segue destinationViewController].
 // Pass the selected object to the new view controller.
 }
 */

@end
