//Copyright © 2015-2017, Verizon and/or its Licensors. All rights reserved.

#import <XCTest/XCTest.h>
#import "PCTestConstants.h"

@interface PCRenameAPITests : XCTestCase

@end

@implementation PCRenameAPITests

- (void)setUp {
    [super setUp];
    // Put setup code here. This method is called before the invocation of each test method in the class.
}

- (void)tearDown {
    // Put teardown code here. This method is called after the invocation of each test method in the class.
    [super tearDown];
}

/*!
 * Test Case: auth token is a required parameter. SDK should throw an exception in case auth token is nil.
 */
- (void)testNilAuthToken {
    // If User is not logged in, copy should throw an exception
    if (![[PCAPISDK sharedInstance] isUserLoggedIn]) {
        
        PCFOPSAPIParams *params = [[PCFOPSAPIParams alloc] init];
        
        XCTAssertThrows([[PCAPISDK sharedInstance] rename:params withCompletionHandler:^(PCResource *metadata, PCAPIQueryResponse *response) {}] , @"Rename with nil access token");
    }
}

/*!
 * Test Case: source is a required parameter. SDK should throw an exception in case source is nil.
 */
- (void)testValidSourceForRenameAPI {
    
    if ([[PCAPISDK sharedInstance] isUserLoggedIn]) {
        PCFOPSAPIParams *params = [[PCFOPSAPIParams alloc] init];
        params.target = @"/VZMOBILE";
        XCTAssertThrows([[PCAPISDK sharedInstance] rename:params withCompletionHandler:^(PCResource *metadata, PCAPIQueryResponse *response) {}] ,@"Rename API without valid source");
    } else {
        XCTFail(@"Test Failed due to nil access token");
    }
}


/*!
 * Test Case: target is a required parameter. SDK should throw an exception in case target is nil.
 */
- (void) testValidTargetForRenameAPI {
    
    if ([[PCAPISDK sharedInstance] isUserLoggedIn]) {
        
        XCTestExpectation *expectation = [self expectationWithDescription:@"Test for valid target"];
        
        PCMetaDataAPIParams *params = [[PCMetaDataAPIParams alloc] init];
        params.path = @"/VZMOBILE";
        
        [[PCAPISDK sharedInstance] fetchMetadata:params withCompletionHandler:^(PCMetadata *metadata, PCAPIQueryResponse *response) {
            switch (response.urlResponse.statusCode) {
                case PCAPIResponseCodeOK: {
                    PCFOPSAPIParams *params = [[PCFOPSAPIParams alloc] init];
                    PCResource *resource = [metadata.subFiles objectAtIndex:0];
                    params.source = resource.fullPath;
                    XCTAssertThrows([[PCAPISDK sharedInstance] rename:params withCompletionHandler:^(PCResource *metadata, PCAPIQueryResponse *response) {}] ,@"Rename without valid target");
                    [expectation fulfill];
                }
                    break;
                    
                default:
                    XCTFail(@"Test could not be completed because metadata API failed");
                    break;
            }
        }];
        [self waitForExpectationsWithTimeout:ExpectationTimeOut*2 handler:^(NSError *error) {
            
            if (error) {
                NSLog(@"Timeout Error: %@", error);
            }
        }];
        
    } else {
        XCTFail(@"Test Failed due to nil access token");
    }
}


/*!
 * Test Case: Check when safe is set true and conflict resolve not set anyvalue , It should return 404 error.
 */

- (void) testWhenConflictResolveNotSetValue {
    
    if ([[PCAPISDK sharedInstance] isUserLoggedIn]) {
        
        XCTestExpectation *expectation = [self expectationWithDescription:@"Test when conflict resolve has invalid value"];
        
        PCMetaDataAPIParams *params = [[PCMetaDataAPIParams alloc] init];
        params.path = @"/VZMOBILE";
        
        [[PCAPISDK sharedInstance] fetchMetadata:params withCompletionHandler:^(PCMetadata *metadata, PCAPIQueryResponse *response) {
            switch (response.urlResponse.statusCode) {
                case PCAPIResponseCodeOK: {
                    PCFOPSAPIParams *params = [[PCFOPSAPIParams alloc] init];
                    PCResource *resource = [metadata.subFiles objectAtIndex:0];
                    params.source = resource.fullPath;
                    params.target = @"/VZMOBILE";
                    params.safe = @"true";
                    
                    [[PCAPISDK sharedInstance] rename:params withCompletionHandler:^(PCResource *metadata, PCAPIQueryResponse *response) {
                        
                        switch (response.urlResponse.statusCode) {
                            case PCAPIResponseCodeConfict:
                                break;
                            default:
                                XCTFail(@"Test case failed. It should return 404 error");
                                break;
                        }
                        [expectation fulfill];
                    }];
                    
                }
                    break;
                    
                default:
                    XCTFail(@"Test could not be completed because metadata API failed");
                    break;
            }
        }];
        
        [self waitForExpectationsWithTimeout:ExpectationTimeOut*2 handler:^(NSError *error) {
            
            if (error) {
                NSLog(@"Timeout Error: %@", error);
            }
        }];
        
    } else {
        XCTFail(@"Test Failed due to nil access token");
    }
}


/*!
 * Test Case: Check when safe is set true and conflict resolve set value as "copy" , It should rename file with new name.
 */

- (void) testWhenConflictResolveSetValue {
    
    if ([[PCAPISDK sharedInstance] isUserLoggedIn]) {
        
        XCTestExpectation *expectation = [self expectationWithDescription:@"Test when conflict resolve has invalid value"];
        
        PCMetaDataAPIParams *params = [[PCMetaDataAPIParams alloc] init];
        params.path = @"/VZMOBILE";
        
        [[PCAPISDK sharedInstance] fetchMetadata:params withCompletionHandler:^(PCMetadata *metadata, PCAPIQueryResponse *response) {
            switch (response.urlResponse.statusCode) {
                case PCAPIResponseCodeOK: {
                    PCFOPSAPIParams *params = [[PCFOPSAPIParams alloc] init];
                    PCResource *resource = [metadata.subFiles objectAtIndex:0];
                    params.source = resource.fullPath;
                    params.target =  resource.fullPath;
                    params.safe = @"true";
                    params.conflictsolve = @"copy";
                    
                    [[PCAPISDK sharedInstance] rename:params withCompletionHandler:^(PCResource *metadataRenamed, PCAPIQueryResponse *response) {
                        
                        switch (response.urlResponse.statusCode) {
                            case PCAPIResponseCodeOK:
                                break;
                            default:
                                XCTFail(@"Test case failed. It should return new renamed file");
                                break;
                        }
                        [expectation fulfill];
                    }];
                    
                }
                    break;
                    
                default:
                    XCTFail(@"Test could not be completed because metadata API failed");
                    break;
            }
        }];
        
        [self waitForExpectationsWithTimeout:ExpectationTimeOut*2 handler:^(NSError *error) {
            
            if (error) {
                NSLog(@"Timeout Error: %@", error);
            }
        }];
        
    } else {
        XCTFail(@"Test Failed due to nil access token");
    }
}

/*!
 * Test Case: If response code is 200,  rename API response contains the file which renamed.
 */
- (void)testToRenameFileToTarget {
    
    if ([[PCAPISDK sharedInstance] isUserLoggedIn]) {
        
        XCTestExpectation *expectation = [self expectationWithDescription:@"Test to rename file to given target"];
        
        PCMetaDataAPIParams *params = [[PCMetaDataAPIParams alloc] init];
        params.path = @"/VZMOBILE";
        
        [[PCAPISDK sharedInstance] fetchMetadata:params withCompletionHandler:^(PCMetadata *metadata, PCAPIQueryResponse *response) {
            switch (response.urlResponse.statusCode) {
                case PCAPIResponseCodeOK: {
                    PCFOPSAPIParams *params = [[PCFOPSAPIParams alloc] init];
                    PCResource *resource = [metadata.subFiles objectAtIndex:0];
                    params.source = resource.fullPath;;
                    params.target =[NSString stringWithFormat:@"%@/%@", @"/VZMOBILE", @"fileRenameTest"];
                    
                    [[PCAPISDK sharedInstance] rename:params withCompletionHandler:^(PCResource *metadata, PCAPIQueryResponse *response) {
                        
                        switch (response.urlResponse.statusCode) {
                            case PCAPIResponseCodeOK: {
                                
                                XCTAssertTrue([metadata.name isEqualToString:@"fileRenameTest"], @"File not renamed");
                            }
                                break;
                            default:
                                XCTFail(@"Test could not be completed because rename API failed");
                                break;
                        }
                        [expectation fulfill];
                    }];
                    
                }
                    break;
                    
                default:
                    XCTFail(@"Test could not be completed because metadata API failed");
                    break;
            }
        }];
        
        [self waitForExpectationsWithTimeout:ExpectationTimeOut*2 handler:^(NSError *error) {
            if (error) {
                NSLog(@"Timeout Error: %@", error);
            }
        }];
        
    } else {
        
        XCTFail(@"Test Failed due to nil access token");
        
    }
    
}

@end
